
// AutomCliView.cpp : implementation of the CAutomCliView class
//

#include "stdafx.h"
#include "AutomCli.h"
#include <vector>

#include "AutomCliDoc.h"
#include "AutomCliView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

HWND g_hCtlIncomming = HWND(0);

// CAutomCliView

IMPLEMENT_DYNCREATE(CAutomCliView, CFormView)

BEGIN_MESSAGE_MAP(CAutomCliView, CFormView)
  ON_CBN_SELENDOK(IDC_CmdText                     , &CAutomCliView::OnCbnCmdTextSelEndOk)
  ON_COMMAND(   IDM_SendManyCmdsToTelemator       , &CAutomCliView::OnSendManyCmdsToTelemator)
  ON_BN_CLICKED(IDC_SendCommandToTelemator        , &CAutomCliView::OnSendCommandToTelemator)
  ON_BN_CLICKED(IDC_SetForegroundWindowTelemator  , &CAutomCliView::OnSetForegroundWindowTelemator)
  ON_BN_CLICKED(IDcb_ComBusyState                 , &CAutomCliView::OnBnClickedCombusystate)
END_MESSAGE_MAP()

// CAutomCliView construction/destruction

CAutomCliView::CAutomCliView()
	: CFormView(CAutomCliView::IDD)
{
	// TODO: add construction code here
}

CAutomCliView::~CAutomCliView()
{
}

void CAutomCliView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
}

BOOL CAutomCliView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CFormView::PreCreateWindow(cs);
}

void CAutomCliView::CmdList_TransferToComboBox(void) {
  HWND hCombo = ::GetDlgItem(m_hWnd, IDC_CmdText);
  ComboBox_ResetContent(hCombo);
  ::SetWindowText(hCombo, _T("Choose one command"));

  for (const auto& it : m_vCmdList)
    ComboBox_AddString( hCombo, it.m_sCmd );
}

void CAutomCliView::OnCbnCmdTextSelEndOk() {
  // Update argument/payload when command is selected from combobox
  HWND hCombo = ::GetDlgItem(m_hWnd, IDC_CmdText);
  int ix = ComboBox_GetCurSel(hCombo);
  if (ix >= 0 && ix<(int)m_vCmdList.size()) {
    if (!m_vCmdList[ix].m_sArg.IsEmpty())
      SetDlgItemText(IDC_CmdArg, m_vCmdList[ix].m_sArg);
  }
}

void CAutomCliView::OnInitialUpdate()
{
	CFormView::OnInitialUpdate();
	GetParentFrame()->RecalcLayout();
	ResizeParentToFit();

  g_hCtlIncomming = ::GetDlgItem(m_hWnd,IDC_Incomming);

  HWND hwndTop = AfxGetMainWnd()->m_hWnd;
  TCHAR szClass[_MAX_PATH]; ::GetClassName(hwndTop, szClass, _countof(szClass));

	CString strServerName; // get server name from doc template string
	GetDocument()->GetDocTemplate()->GetDocString(strServerName, CDocTemplate::regFileTypeId);

  TCHAR szDDEApplication[_MAX_PATH]; VERIFY(GlobalGetAtomName(AfxGetApp()->m_atomApp, szDDEApplication, _countof(szDDEApplication)) != 0);
  TCHAR szDDETopic[_MAX_PATH];       VERIFY(GlobalGetAtomName(AfxGetApp()->m_atomSystemTopic, szDDETopic,_countof(szDDETopic)) != 0);

  CString sInfo;
  sInfo.Format( _T("[ExecuteFromTelemator]\r\n")
                _T("UseAutomation=1 ; 0 if you want to DDE instead\r\n")
                _T("ProgID=%s\r\n")
                _T("DDEApplication=%s\r\n")
                _T("DDETopic=%s\r\n")
                _T("WindowClassName=%s"),
                   (LPCTSTR)strServerName,
                   szDDEApplication, szDDETopic,
                   szClass );
  SetDlgItemText( IDC_ConfigInfo, sInfo );

  CmdList_FillDefault();
  CmdList_TransferToComboBox();
}


// CAutomCliView diagnostics

#ifdef _DEBUG
void CAutomCliView::AssertValid() const
{
	CFormView::AssertValid();
}

void CAutomCliView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CAutomCliDoc* CAutomCliView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CAutomCliDoc)));
	return (CAutomCliDoc*)m_pDocument;
}
#endif //_DEBUG


// CAutomCliView message handlers
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// Automation Client starts here:

bool IsTelematorRunning(void) {
  HWND hWnd = FindWindow(_T("TelematMainWClass"), NULL );
  return IsWindow(hWnd)!=0;
}

class ITelemator : public COleDispatchDriver
{
public:
	ITelemator() {}		// Calls COleDispatchDriver default constructor
	ITelemator(LPDISPATCH pDispatch) : COleDispatchDriver(pDispatch) {}
	ITelemator(const ITelemator& dispatchSrc) : COleDispatchDriver(dispatchSrc) {}
  bool MyGetObject(void);
  bool MyGetDispId( LPCTSTR pszName, DISPID& outId );
public:
  CString ExecuteFromGis(LPCTSTR pszCmd, LPCTSTR pszArg);
  CString ExecuteHttpJson(LPCTSTR pszMethod, LPCTSTR pszCmd, LPCTSTR pszArg);
  HWND    GetForegroundHwnd(void);
};

bool ITelemator::MyGetObject(void) {
  CString sOutError = _T("GetTelematorObject failed.\n");
  IDispatch *pDispatch=NULL;

  // Translate server ProgID into a CLSID. ClsidFromProgID
  // gets this information from the registry.
  CLSID clsid;
	HRESULT hr = CLSIDFromProgID(OLESTR("Telemator.Document"), &clsid);
	if( FAILED(hr) ) {
	  sOutError += _T("CLSIDFromProgID() failed\n")
                 _T("Does registry key HKEY_CLASSES_ROOT\\Telemator.Document\\CLSID exist?\n")
                 _T("(If not, you can create the key from the Telemator Help-menu.)");
	  goto lblError;
	}

  {
    IUnknown  *pUnknown; // Get an interface to the running instance, if any..
    hr = GetActiveObject(clsid, NULL, (IUnknown**)&pUnknown);
    if( FAILED(hr) ) {
      sOutError += _T("GetActiveObject() failed\n");
      goto lblError;
    }

    // Get IDispatch interface for Automation...
    hr = pUnknown->QueryInterface(IID_IDispatch, (void **)&pDispatch);
    pUnknown->Release(); pUnknown=NULL; // Release the no-longer-needed IUnknown
    if( FAILED(hr) ) goto lblError;
    if( FAILED(hr) ) {
      sOutError += _T("pUnknown->QueryInterface(IID_IDispatch) failed\n");
      goto lblError;
    }
  }

  this->AttachDispatch(pDispatch, TRUE);
  if( this->m_lpDispatch == NULL ) goto lblError;
  return true;

lblError:
  if( !IsTelematorRunning() ) sOutError += _T("\n\nIs Telemator running?");
  AfxMessageBox( sOutError );
  return false;
}

bool ITelemator::MyGetDispId( LPCTSTR pszName, DISPID& outId ) {
  if( m_lpDispatch!=NULL ) {
    USES_CONVERSION;
    LPCOLESTR lpOleStr = T2COLE(pszName);
    HRESULT hr = m_lpDispatch->GetIDsOfNames(IID_NULL, (LPOLESTR *)&lpOleStr, 1,
                                             LOCALE_USER_DEFAULT, &outId );
    if( !FAILED(hr) ) return true;
  }
  return false;
}

CString ITelemator::ExecuteFromGis(LPCTSTR pszCmd, LPCTSTR pszArg)
{
  DISPID dispid;
  if( !MyGetDispId( _T("ExecuteFromGis"), dispid ) )
    return _T("-1 Method not found");

  CString sResult;
  static BYTE parms[] = VTS_BSTR VTS_BSTR;
  InvokeHelper( dispid, DISPATCH_METHOD, VT_BSTR,
               (void*)&sResult /*function expects CString for VT_BSTR*/,
                parms, pszCmd, pszArg);

  return sResult;
}

CString ITelemator::ExecuteHttpJson(LPCTSTR pszMethod /*GET/PUT/POST*/, LPCTSTR pszUri, LPCTSTR pszJson)
{
  DISPID dispid;
  if( !MyGetDispId( _T("ExecuteHttpJson"), dispid ) )
    return _T("-1 Method not found");

  CString sResult;
  static BYTE parms[] = VTS_BSTR VTS_BSTR VTS_BSTR;
  InvokeHelper( dispid, DISPATCH_METHOD, VT_BSTR,
               (void*)&sResult /*function expects CString for VT_BSTR*/,
                parms, pszMethod, pszUri, pszJson);

  return sResult;
}

HWND ITelemator::GetForegroundHwnd(void)
{
  DISPID dispid;
  if( !MyGetDispId( _T("GetForegroundHwnd"), dispid ) )
    return 0; // _T("-1 Method not found");

  HWND hwndResult;
  InvokeHelper( dispid, DISPATCH_METHOD,  VT_I4, (void*)& hwndResult, NULL );
  return  hwndResult;
}

//-----------------------------------------------------

bool MyInStr( CString sSrc, CString sFind ) {
  sSrc.MakeUpper(); sFind.MakeUpper();
  return sSrc.Find( sFind )>=0;
}

CString Test_GetObjectExecuteFromGis( const CString& strCmd, const CString& strArg ) {
  ITelemator objTel;
  if( objTel.MyGetObject() )
    return objTel.ExecuteFromGis( strCmd, strArg );
  else
    return _T("objTel.MyGetObject() failed");
}

CString Test_GetObjectExecuteHttpJson( const CString& strMethod, const CString& strCmd, const CString& strArg ) {
  ITelemator objTel;
  if( objTel.MyGetObject() )
    return objTel.ExecuteHttpJson( strMethod, strCmd, strArg );
  else
    return _T("objTel.MyGetObject() failed");
}

void CAutomCliView::UpdateResultWindows( const CString& strRes, TCHAR chSplitResult )
{
  // Split result-string
  CString strResCode=strRes, strResText;
  int i = strRes.Find( chSplitResult );
  if( i>=0 ) {
    strResCode = strRes.Left( i );
    strResText = strRes.Mid( i+1 );
    // Edit control must have \r\n - not only \n
    strResText.Replace( _T("\r"), _T("") );
    strResText.Replace( _T("\n"), _T("\r\n") );
  }
  SetDlgItemText( IDC_ResultCode, strResCode );
  SetDlgItemText( IDC_ResultText, strResText );
}

void CAutomCliView::OnSendCommandToTelematorHttpJson()
{
  ITelemator objTel;
  if( objTel.MyGetObject() ) {
    CString strCmd, strMethod, strUrl, strJson, strRes;

    GetDlgItemText( IDC_CmdText, strCmd );
    GetDlgItemText( IDC_CmdArg , strJson );

    AfxExtractSubString( strMethod, strCmd, 1, _T('/') );
    int        i=strCmd.Find(_T('/'));
    if( i>=0 ) i=strCmd.Find(_T('/'),i+1);
    strUrl = strCmd.Mid(i);

    strJson.Replace( _T("\\r"), _T("\r") );
    strJson.Replace( _T("\\n"), _T("\n") );

    SetDlgItemText( IDC_ResultCode, _T("") );
    SetDlgItemText( IDC_ResultText, _T("") );
    SetDlgItemText( IDC_Incomming,  _T("") );

    strRes = objTel.ExecuteHttpJson( strMethod, strUrl, strJson );
    UpdateResultWindows( strRes, _T('\n') );

    if( _ttoi(strRes) >=0 ) // Success - Set Telemator as foreground-window
      OnSetForegroundWindowTelemator();
  }
}

void CAutomCliView::OnSendCommandToTelematorFromGis()
{
  ITelemator objTel;
  if( objTel.MyGetObject() ) {
    CString strCmd, strArg, strRes;

    GetDlgItemText( IDC_CmdText, strCmd );
    GetDlgItemText( IDC_CmdArg, strArg );

    { // Remove from: (
      int i= strCmd.Find( _T('(') );
      if( i>=0 ) strCmd = strCmd.Left(i);
      strCmd.TrimRight();
    }
    strArg.Replace( _T(";")  , _T("\t") );
    strArg.Replace( _T("\\r"), _T("\r") );
    strArg.Replace( _T("\\n"), _T("\n") );

    SetDlgItemText( IDC_ResultCode, _T("") );
    SetDlgItemText( IDC_ResultText, _T("") );
    SetDlgItemText( IDC_Incomming,  _T("") );

    strRes = objTel.ExecuteFromGis( strCmd, strArg );
    UpdateResultWindows( strRes, _T('\t') );

    if( MyInStr( strCmd, _T("SetForegroundWindowGIS"))) return;
    if( MyInStr( strCmd, _T("GetAdoConnectString"   ))) return;
    if( MyInStr( strCmd, _T("GetOdbcConnectString"  ))) return;

    if( _ttoi(strRes) >=0 ) // Success - Set Telemator as foreground-window
      OnSetForegroundWindowTelemator();
  }
}

void CAutomCliView::OnSendCommandToTelemator()
{
  CString strCmd; GetDlgItemText( IDC_CmdText, strCmd );
  strCmd.Trim();
  if( *(LPCTSTR)strCmd == _T('/') )
    OnSendCommandToTelematorHttpJson();
  else
    OnSendCommandToTelematorFromGis();
}

void CAutomCliView::OnSendManyCmdsToTelemator()
{
  std::vector<CString> as;
  CString sFname = _T("cmds.txt");

  { FILE *in;
    if( _tfopen_s( &in, sFname, _T("rt") ) != 0 )
    { AfxMessageBox(_T("Cannot read from file ")+sFname); return; }
    TCHAR sz[1024*5];
    while( _fgetts( sz, _countof(sz), in)!=NULL ) {
      CString s = sz; s.TrimRight(_T("\r\n ") );
      s.Replace( _T("\\t"), _T("\t") );
      s.Replace( _T(';'), _T('\t') );
      if( !s.IsEmpty() )
        as.push_back( s );
    }
    fclose( in );
  }

  for( size_t i=0; i<as.size(); ++i ) {
    CString sCmd, sArg;
    sCmd = as[i];
    int ix = sCmd.Find(_T('\t'));
    if( ix>0 ) {
      sArg = sCmd.Mid( ix+1,999 );
      sCmd.Delete(     ix  ,999 );

      SetDlgItemText( IDC_CmdText, sCmd );
      SetDlgItemText( IDC_CmdArg,  sArg );

      OnSendCommandToTelemator();
    }
  }
}

void CAutomCliView::OnSetForegroundWindowTelemator()
{
  ITelemator objTel;
  if( objTel.MyGetObject() ) {
    HWND hWndTop = objTel.GetForegroundHwnd();
    if( IsWindow(hWndTop) )
      ::SetForegroundWindow( hWndTop );
  }
}

void CAutomCliView::OnBnClickedCombusystate()
{
  static BOOL bOn = false;
  COleMessageFilter* pMessageFilter = AfxOleGetMessageFilter();
  if( pMessageFilter!=NULL ) {
    pMessageFilter->SetBusyReply( SERVERCALL_RETRYLATER);

    BOOL bCheck = IsDlgButtonChecked( IDcb_ComBusyState )!=0;
    if( bOn!=bCheck ) {
      if( bCheck ) pMessageFilter->BeginBusyState();
      else         pMessageFilter->EndBusyState();
      bOn = bCheck;
    }
  }
}

static CAutomCliView::CElmCmdList DecodeCmd2CmdPayload(const CString& sIn) {
  CAutomCliView::CElmCmdList elm;

  int ixPayload = sIn.Find('{'); // Find json payload start
  if (ixPayload>0) {
    elm.m_sCmd = sIn.Left(ixPayload - 1);
    elm.m_sArg = sIn.Mid(ixPayload);
  }
  else
    elm.m_sCmd = sIn;

  elm.m_sCmd.TrimRight(_T(" \t\r\n"));
  elm.m_sArg.TrimRight(_T(" \t\r\n"));

  return elm;
}

void CAutomCliView::CmdList_FillDefault(void)
{
  m_vCmdList.clear();
  m_vCmdList.reserve(500);

  m_vCmdList.emplace_back(_T("ShowCable  (cable id)"));
  m_vCmdList.emplace_back(_T("ShowCableZoom  (cable id)"));
  m_vCmdList.emplace_back(_T("ShowCableTrace  (trace id)"));
  m_vCmdList.emplace_back(_T("ShowEnd  (point or equipment id)"));
  m_vCmdList.emplace_back(_T("ShowEndZoom  (point or equipment id)"));
  m_vCmdList.emplace_back(_T("ShowCirc  (circuit id)"));
  m_vCmdList.emplace_back(_T("ShowCustomer  (customer id)"));
  m_vCmdList.emplace_back(_T(""));
  m_vCmdList.emplace_back(_T("ShowEquipm  (equipment id)"));
  m_vCmdList.emplace_back(_T("ShowEquipmZoomCon  (equipment id)"));
  m_vCmdList.emplace_back(_T("ShowEquipmZoomTrn  (equipment id)"));
  m_vCmdList.emplace_back(_T("ShowPoint  (point id)"));
  m_vCmdList.emplace_back(_T("ShowPointZoom  (point id)"));
  m_vCmdList.emplace_back(_T(""));
  m_vCmdList.emplace_back(_T("ShowEndsWithCables ( end <tab> end <tab> end ...)"));
  m_vCmdList.emplace_back(_T("ShowEndsWithCableTraces ( end <tab> end <tab> end ...)"));
  m_vCmdList.emplace_back(_T("ShowCircInNetdiagDirect (circuit id)"));
  m_vCmdList.emplace_back(_T(""));
  m_vCmdList.emplace_back(_T("ShowCableAskDelete      (cable id)"));
  m_vCmdList.emplace_back(_T("ShowCableTraceAskDelete (trace id)"));
  m_vCmdList.emplace_back(_T("ShowCableTraceAskMerge  (trace id 1, trace id 2)"));
  m_vCmdList.emplace_back(_T("ShowEndAskDelete        (point or equipment id)"));
  m_vCmdList.emplace_back(_T("ShowEndAskMerge         (point id 1, point id 2)"));
  m_vCmdList.emplace_back(_T("ShowCustomerAskDelete   (customer id)"));
  m_vCmdList.emplace_back(_T(""));
  m_vCmdList.emplace_back(_T("InsertEnd  (point or equipment id)"));
  m_vCmdList.emplace_back(_T("ModifyEnd  (point or equipment id)"));
  m_vCmdList.emplace_back(_T("RenameEnd  (old point or equipment id <tab> new point or equipment id)"));
  m_vCmdList.emplace_back(_T("CopyEnd    (source point or equipment id <tab> new point/equipment id)"));
  m_vCmdList.emplace_back(_T(""));

  // Powel Explorer and TM-GIS uses theese - (Cable traces stored in Telemator)
  m_vCmdList.emplace_back(_T("InsertCableTrace      (trace id <tab> end A <tab> end B <tab> fields=)"));
  m_vCmdList.emplace_back(_T("ModifyCableTrace      (trace id <tab> end A <tab> end B <tab> fields=)"));
  m_vCmdList.emplace_back(_T("RenameCableTrace      (old trace id <tab> new trace id)"));
  m_vCmdList.emplace_back(_T("CopyCableTrace        (src trace1 id <tab> dst trace2 id <tab> trace2 end A <tab> trace2 end B <tab> trace2 length)"));
  m_vCmdList.emplace_back(_T("SplitCableTraceInPoint(src trace1 id <tab> dst trace2 id <tab> split in point <tab> trace1 length <tab> trace2 length)"));
  //ComboBox_AddString( hCombo, _T("SplitCableTrace       (src trace1 id <tab> dst trace2 id <tab> trace1 end A <tab> trace1 end B <tab> trace2 end A <tab> trace2 end B <tab> trace1 length <tab> trace2 length)"));
  m_vCmdList.emplace_back(_T("SpliceDuct            (end <tab> trace1 id <tab> duct1 id <tab> trace2 id <tab> duct2 id <tab> splice type)"));
  m_vCmdList.emplace_back(_T("PutCableIntoTraces    (tab separeted list of trace ids)"));

  // TelMe uses this
  m_vCmdList.emplace_back(_T("PutDuctIntoTraces  (trace template <tab> end A <tab> end B <tab> tab separeted list of trace ids)"));
  m_vCmdList.emplace_back(_T("SpliceDuctDlg (point id )"));
  m_vCmdList.emplace_back(_T(""));

  //  TM-GIS uses this
  m_vCmdList.emplace_back(_T("PutOneCableIntoTrace ( cable id <tab> trace id <tab> duct id)"));
  m_vCmdList.emplace_back(_T(""));

  //  NetBas/OpenNIS might use theese instead - (Cable traces stored in external app)
  m_vCmdList.emplace_back(_T("InsertCable      (cable id <tab> end A <tab> end B <tab> fields=)"));
  m_vCmdList.emplace_back(_T("ModifyCable      (cable id <tab> end A <tab> end B <tab> fields=)"));
  m_vCmdList.emplace_back(_T("RenameCable      (old cable id <tab> new cable id)"));
  m_vCmdList.emplace_back(_T("CopyCable        (src cable1 id <tab> dst cable2 id <tab> cable2 end A <tab> cable2 end B <tab> cable2 length)"));
  m_vCmdList.emplace_back(_T("SplitCableInPoint(src cable1 id <tab> dst cable2 id <tab> split in point <tab> cable1 length <tab> cable2 length)"));
  //m_vCmdList.emplace_back(_T("SplitCable       (src cable1 id <tab> dst cable2 id <tab> cable1 end A <tab> cable1 end B <tab> cable2 end A <tab> cable2 end B <tab> cable1 length <tab> cable2 length)"));
  m_vCmdList.emplace_back(_T(""));


  //  Powel NetBas uses theese
  m_vCmdList.emplace_back(_T("InsertCustomer  (customer id <tab> fields=)"));
  m_vCmdList.emplace_back(_T("ModifyCustomer  (customer id <tab> fields=)"));

  // For future use
  m_vCmdList.emplace_back(_T(""));
  m_vCmdList.emplace_back(_T("InsertCircuit (circuit id <tab> start-end <tab> stop-end <tab> fields=)"));
  m_vCmdList.emplace_back(_T("InsertAndRoutCircuit ( start-end <tab> stop-end )"));
  m_vCmdList.emplace_back(_T("ShowCircAskAssignStartStopEnd( circuit id <tab> start-end <tab> stop-end )"));
  m_vCmdList.emplace_back(_T("SetDefaultProject( project id )"));
  m_vCmdList.emplace_back(_T("ImportTablesFromFile( filename <tab> flag )"));


  // TelMe uses theese
  m_vCmdList.emplace_back(_T(""));
  m_vCmdList.emplace_back(_T("PrintSelected( <LF>tab separeted list of point ids <LF>tab separeted list of trace ids <LF>tab separeted list of cable ids)"));
  // m_vCmdList.emplace_back(_T("PrintNumCoresDirectlyBetweenPoints  (tab separeted list of point ids)"));

  //  When GIS wants to be on top
  m_vCmdList.emplace_back(_T(""));
  m_vCmdList.emplace_back(_T("SetForegroundWindowGIS"));

  // Powel NetBase uses theese
  m_vCmdList.emplace_back(_T(""));
  m_vCmdList.emplace_back(_T("OpenNetworkAreaInFolder ( path )"));
  m_vCmdList.emplace_back(_T("CreateNetworkAreaInFolder ( path, flags )"));
  m_vCmdList.emplace_back(_T("MergeNetworkAreaInFolder ( path )"));


  //  Ask Telemator for information / DSN less database connection
  m_vCmdList.emplace_back(_T(""));
  m_vCmdList.emplace_back(_T("GetOdbcConnectString"));
  m_vCmdList.emplace_back(_T("GetAdoConnectString"));

  m_vCmdList.emplace_back(_T("GetActiveCable"));
  m_vCmdList.emplace_back(_T("GetCableStatusText ( cable id )"));
  m_vCmdList.emplace_back(_T("GetCableCalculatedCoreinfo ( cable id(s) )"));
  m_vCmdList.emplace_back(_T("GetActiveCirc"));
  m_vCmdList.emplace_back(_T("GetUncutAvailableDuctsFromPoint ( point id )"));
  m_vCmdList.emplace_back(_T("GetNextFreeId ( table[cable,trace,point,circuit,project] <tab> wanted id )"));
  m_vCmdList.emplace_back(_T("IsCableTraceModuleInstalled"));
  m_vCmdList.emplace_back(_T(""));
  m_vCmdList.emplace_back(_T(""));

  { // Fill extra commands from textfile:
    CString sPath;
    for(int i=1; i<__argc; ++i ) {
      CString sArg = __targv[i]; 
      int ixSep = sArg.Find('=');
      if( ixSep>0 ) {
        CString s0 = sArg.Left(ixSep+1);
        if( s0.CompareNoCase(_T("-ExtraCmd="))==0 ) 
          sPath = sArg.Mid(ixSep+1);
      }   
    }
    if( !sPath.IsEmpty() ) {
      FILE *stream = NULL;
      if (0 != _wfopen_s(&stream, sPath, _T("rt, ccs=UTF-8"))) 
        AfxMessageBox( _T("Cannot open file:\n") + sPath );
      else {
        CStdioFile f(stream); CString sBuf;
        while( f.ReadString(sBuf) ) {
          sBuf.TrimRight(_T(" \r\n"));
          m_vCmdList.push_back(DecodeCmd2CmdPayload(sBuf));
        }
        f.Close();
      }
    }
  }
}
